@@ -1,5 +1,5 @@
 package Test::BDD::Cucumber::Errors;
-$Test::BDD::Cucumber::Errors::VERSION = '0.31';
+$Test::BDD::Cucumber::Errors::VERSION = '0.45';
 use strict;
 use warnings;
 
@@ -13,7 +13,7 @@ Test::BDD::Cucumber::Errors - Consistently formatted errors
 
 =head1 VERSION
 
-version 0.31
+version 0.45
 
 =head1 DESCRIPTION
 
@@ -27,6 +27,10 @@ practice are errors with the input test scenarios, and it's helpful to have the
 location of the error and context when debugging those. Perhaps in the future
 these can return error objects.
 
+All current uses (2016-02-09) just pass the results straight to die, so I
+have decided to UTF8 encode the error message on the basis that this probably
+constitutes an application boundary.
+
 =head1 SYNOPSIS
 
   use Test::BDD::Cucumber::Errors qw/parse_error_from_line/;
@@ -63,7 +67,7 @@ sub parse_error_from_line {
     # Get the caller data
     my ( $caller_filename, $caller_line ) = ( caller() )[ 1, 2 ];
 
-    # Get the simplistic filename and line number it occured on
+    # Get the simplistic filename and line number it occurred on
     my $feature_filename = $line->document->filename || "(no filename)";
     my $feature_line = $line->number;
 
@@ -79,13 +83,14 @@ sub parse_error_from_line {
           sprintf( "% 3d%s    %s\n", $actual_line, $mark, $lines[$_] );
     }
 
-    return (
-        sprintf( $error,
-            $feature_filename, $feature_line,
-            $caller_filename,  $caller_line,
-            $feature_filename, $formatted_lines,
-            ( '-' x ( ( length $feature_filename ) + 8 ) ) )
-    );
+    my $to_return = sprintf( $error,
+        $feature_filename, $feature_line,
+        $caller_filename,  $caller_line,
+        $feature_filename, $formatted_lines,
+        ( '-' x ( ( length $feature_filename ) + 8 ) ) );
+
+    utf8::encode($to_return);
+    return $to_return;
 }
 
 sub _get_context_range {
