@@ -1,12 +1,12 @@
 package Test::BDD::Cucumber::Executor;
-$Test::BDD::Cucumber::Executor::VERSION = '0.31';
+$Test::BDD::Cucumber::Executor::VERSION = '0.45';
 =head1 NAME
 
 Test::BDD::Cucumber::Executor - Run through Feature and Harness objects
 
 =head1 VERSION
 
-version 0.31
+version 0.45
 
 =head1 DESCRIPTION
 
@@ -16,19 +16,88 @@ Definitions, and reporting on progress through the passed-in harness.
 =cut
 
 use Moose;
-use Storable qw(dclone);
+use Clone qw(clone);
 use List::Util qw/first/;
 use List::MoreUtils qw/pairwise/;
-use Test::Builder;
+use Module::Runtime qw/use_module/;
 use Number::Range;
+use Carp qw/croak/;
+our @CARP_NOT;
+
+use Test::Builder;
+
+# Setup wrapping for Test::Builder
+use Test::BDD::Cucumber::TestBuilderDelegator;
+use Devel::Refcount qw/refcount/;
+if ( ( !$ENV{'TEST_BDD_CUCUMBER_NO_TB_WRAP_TEST'} )
+    && refcount($Test::Builder::Test) > 1 )
+{
+
+    my $ref_trace = "[Install Devel::FindRef to see these diagnostics]";
+    if ( eval { use_module "Devel::FindRef" } ) {
+        $ref_trace = Devel::FindRef::track($Test::Builder::Test);
+    }
+
+    my $message = sprintf( <<'END', $ref_trace );
+!!! HEY YOU !!!
+Test::BDD::Cucumber needs to be able to wrap $Test::Builder::Test in order to
+properly capture testing output. However, something else has already taken a
+reference to that module. You need to `use` Test::BDD::Cucumber::Executor
+before the other testing modules are used. Modules that appear to already have a
+reference are:
+-----
+%s
+-----
+You can safetly ignore the global $Test::Builder::Test. In almost all cases, the
+simple fix is the move the line that says `use Test::BDD::Cucumber::Executor`
+above all other `use Test::*` lines. You can also suppress this check by setting:
+
+  TEST_BDD_CUCUMBER_NO_TB_WRAP_TEST=1
+END
+
+    croak $message;
+}
+
+$Test::Builder::Test =
+  Test::BDD::Cucumber::TestBuilderDelegator->new( Test::Builder->new() );
 
 use Test::BDD::Cucumber::StepContext;
 use Test::BDD::Cucumber::Util;
 use Test::BDD::Cucumber::Model::Result;
 use Test::BDD::Cucumber::Errors qw/parse_error_from_line/;
 
+has '_bail_out' => ( is => 'rw', isa => 'Bool', default => 0 );
+
 =head1 METHODS
 
+=head2 extensions
+
+=head2 add_extensions
+
+The attributes C<extensions> is an arrayref of
+L<Test::BDD::Cucumber::Extension> extensions. Extensions have their
+hook-functions called by the Executor at specific points in the BDD feature
+execution.
+
+B<C<<add_extensions>> adds items in FIFO using unshift()>, and are called in
+reverse order at the end hook; this means that if you:
+
+  add_extensions( 1 );
+  add_extensions( 2, 3 );
+
+The C<pre_*> will be called in order 2, 3, 1, and C<post_*> will be called in
+1, 3, 2.
+
+=cut
+
+has extensions => (
+    is      => 'ro',
+    isa     => 'ArrayRef',
+    traits  => ['Array'],
+    default => sub { [] },
+    handles => { add_extensions => 'unshift' },
+);
+
 =head2 steps
 
 =head2 add_steps
@@ -102,6 +171,7 @@ sub execute {
         @scenarios = $tag_spec->filter(@scenarios);
     }
 
+    $_->pre_feature( $feature, $feature_stash ) for @{ $self->extensions };
     for my $scenario (@scenarios) {
 
         # Execute the scenario itself
@@ -115,6 +185,8 @@ sub execute {
             }
         );
     }
+    $_->post_feature( $feature, $feature_stash, 'no' )
+      for reverse @{ $self->extensions };
 
     $harness->feature_done($feature);
 }
@@ -159,7 +231,8 @@ sub execute_scenario {
     my $harness_start = $is_background ? 'background'      : 'scenario';
     my $harness_stop  = $is_background ? 'background_done' : 'scenario_done';
 
-    my $outline_stash = $incoming_outline_stash || { short_circuit => 0 };
+    my $outline_stash = $incoming_outline_stash || {};
+    $outline_stash->{'short_circuit'} ||= $self->_bail_out;
 
     # Multiply out Scenario Outlines as appropriate
     my @datasets = @{ $outline->data };
@@ -168,6 +241,7 @@ sub execute_scenario {
     my $scenario_stash = $incoming_scenario_stash || {};
 
     my %context_defaults = (
+        executor => $self,    # Held weakly by StepContext
 
         # Data portion
         data  => '',
@@ -195,22 +269,22 @@ sub execute_scenario {
             $scenario_stash->{'longest_step_line'} );
 
         if ( not $is_background ) {
+            $_->pre_scenario( $outline, $feature_stash, $scenario_stash )
+              for @{ $self->extensions };
+
             for my $before_step ( @{ $self->{'steps'}->{'before'} || [] } ) {
 
                 # Set up a context
                 my $context = Test::BDD::Cucumber::StepContext->new(
-                    {
-                        %context_defaults, verb => 'before',
-                    }
-                );
+                    { %context_defaults, verb => 'before', } );
 
                 my $result =
                   $self->dispatch( $context, $before_step,
-                    $outline_stash->{'short_circuit'} );
+                    $outline_stash->{'short_circuit'}, 0 );
 
                 # If it didn't pass, short-circuit the rest
                 unless ( $result->result eq 'passing' ) {
-                    $outline_stash->{'short_circuit'}++;
+                    $outline_stash->{'short_circuit'} = 1;
                 }
             }
         }
@@ -245,7 +319,7 @@ sub execute_scenario {
                     # Data portion
                     columns => $step->columns || [],
                     data => ref( $step->data )
-                    ? dclone( $step->data )
+                    ? clone( $step->data )
                     : $step->data || '',
 
                     # Step-specific info
@@ -257,7 +331,7 @@ sub execute_scenario {
 
             my $result =
               $self->find_and_dispatch( $context,
-                $outline_stash->{'short_circuit'} );
+                $outline_stash->{'short_circuit'}, 0 );
 
             # If it didn't pass, short-circuit the rest
             unless ( $result->result eq 'passing' ) {
@@ -271,14 +345,15 @@ sub execute_scenario {
 
                 # Set up a context
                 my $context = Test::BDD::Cucumber::StepContext->new(
-                    {
-                        %context_defaults, verb => 'after',
-                    }
-                );
+                    { %context_defaults, verb => 'after', } );
 
                 # All After steps should happen, to ensure cleanup
-                my $result = $self->dispatch( $context, $after_step, 0 );
+                my $result = $self->dispatch( $context, $after_step, 0, 0 );
             }
+            $_->post_scenario( $outline, $feature_stash, $scenario_stash,
+                $outline_stash->{'short_circuit'} )
+              for reverse @{ $self->extensions };
+
         }
 
         $harness->$harness_stop( $outline, $dataset );
@@ -311,16 +386,17 @@ the steps that have been added to the executor object, executing against the
 first matching one.
 
 You can also pass in a boolean 'short-circuit' flag if the Scenario's remaining
-steps should be skipped.
+steps should be skipped, and a boolean flag to denote if it's a redispatched
+step.
 
 =cut
 
 sub find_and_dispatch {
-    my ( $self, $context, $short_circuit ) = @_;
+    my ( $self, $context, $short_circuit, $redispatch ) = @_;
 
     # Short-circuit if we need to
     return $self->skip_step( $context, 'pending',
-        "Short-circuited from previous tests" )
+        "Short-circuited from previous tests", 0 )
       if $short_circuit;
 
     # Try and find a matching step
@@ -329,13 +405,21 @@ sub find_and_dispatch {
       @{ $self->{'steps'}->{'step'} || [] };
 
     # Deal with the simple case of no-match first of all
-    return $self->skip_step( $context, 'undefined',
+    unless ($step) {
+        my $message =
             "No matching step definition for: "
           . $context->verb . ' '
-          . $context->text )
-      unless $step;
+          . $context->text;
+        my $result =
+          $self->skip_step( $context, 'undefined', $message, $redispatch );
+        return $result;
+    }
 
-    return $self->dispatch( $context, $step, 0 );
+    $_->pre_step( $step, $context ) for @{ $self->extensions };
+    my $result = $self->dispatch( $context, $step, 0, $redispatch );
+    $_->post_step( $step, $context, $result eq 'passing' )
+      for reverse @{ $self->extensions };
+    return $result;
 }
 
 =head2 dispatch
@@ -349,16 +433,17 @@ steps should be skipped.
 =cut
 
 sub dispatch {
-    my ( $self, $context, $step, $short_circuit ) = @_;
+    my ( $self, $context, $step, $short_circuit, $redispatch ) = @_;
 
     return $self->skip_step( $context, 'pending',
-        "Short-circuited from previous tests" )
+        "Short-circuited from previous tests", $redispatch )
       if $short_circuit;
 
     # Execute the step definition
     my ( $regular_expression, $coderef ) = @$step;
 
-    # Setup what we'll pass to step_done, with out localized Test::Builder stuff
+    # Setup what we'll pass to step_done, with out localized Test::Builder
+    # stuff
     my $output    = '';
     my $tb_return = {
         output  => \$output,
@@ -370,12 +455,15 @@ sub dispatch {
     $tb_return->{'builder'}->failure_output( \$output );
     $tb_return->{'builder'}->todo_output( \$output );
 
-    # Make a minumum pass
+    # Make a minimum pass
     $tb_return->{'builder'}
       ->ok( 1, "Starting to execute step: " . $context->text );
 
+    my $step_name = $redispatch ? 'sub_step' : 'step';
+    my $step_done_name = $step_name . '_done';
+
     # Say we're about to start it up
-    $context->harness->step($context);
+    $context->harness->$step_name($context);
 
     # Store the string position of matches for highlighting
     my @match_locations;
@@ -384,36 +472,41 @@ sub dispatch {
     my $result;
     {
         # Localize test builder
-        local $Test::Builder::Test = $tb_return->{'builder'};
+        local $Test::Builder::Test->{'_wraps'} = $tb_return->{'builder'};
+
+        no warnings 'redefine';
+        local *Test::Builder::BAIL_OUT = sub {
+            my ( $tb, $message ) = @_;
+            $self->_bail_out(1);
+            local @CARP_NOT = qw(Test::More Test::BDD::Cucumber::Executor);
+            croak("BAIL_OUT() called: $message");
+        };
 
         # Execute!
-        eval {
-            no warnings 'redefine';
 
-            # Set S and C to be step-specific values before executing the step
-            local *Test::BDD::Cucumber::StepFile::S = sub {
-                return $context->stash->{'scenario'};
-            };
-            local *Test::BDD::Cucumber::StepFile::C = sub {
-                return $context;
-            };
-
-            # Take a copy of this. Turns out actually matching against it
-            # directly causes all sorts of weird-ass eisenbugs which mst has
-            # promised to investigate.
-            my $text = $context->text;
-
-            # Save the matches
-            $context->matches( [ $text =~ $regular_expression ] );
-
-            # Save the location of matched subgroups for highlighting hijinks
-            my @starts = @-;
-            my @ends   = @+;
-            @match_locations = pairwise { [ $a, $b ] } @starts, @ends;
-
-            # OK, actually execute
-            $coderef->($context);
+        # Set S and C to be step-specific values before executing the step
+        local *Test::BDD::Cucumber::StepFile::S = sub {
+            return $context->stash->{'scenario'};
         };
+        local *Test::BDD::Cucumber::StepFile::C = sub {
+            return $context;
+        };
+
+        # Take a copy of this. Turns out actually matching against it
+        # directly causes all sorts of weird-ass heisenbugs which mst has
+        # promised to investigate.
+        my $text = $context->text;
+
+        # Save the matches
+        $context->matches( [ $text =~ $regular_expression ] );
+
+        # Save the location of matched subgroups for highlighting hijinks
+        my @starts = @-;
+        my @ends   = @+;
+        @match_locations = pairwise { [ $a, $b ] } @starts, @ends;
+
+        # OK, actually execute
+        eval { $coderef->($context) };
         if ($@) {
             $Test::Builder::Test->ok( 0, "Test compiled" );
             $Test::Builder::Test->diag($@);
@@ -431,7 +524,6 @@ sub dispatch {
                 output => $output
             }
         );
-
     }
 
     my @clean_matches =
@@ -441,8 +533,8 @@ sub dispatch {
     # Say the step is done, and return the result. Happens outside
     # the above block so that we don't have the localized harness
     # anymore...
-    $context->harness->add_result($result);
-    $context->harness->step_done( $context, $result, \@clean_matches );
+    $context->harness->add_result($result) unless $redispatch;
+    $context->harness->$step_done_name( $context, $result, \@clean_matches );
     return $result;
 }
 
@@ -545,10 +637,13 @@ Harness's step start and step_done methods, and returns a skipped-test result.
 =cut
 
 sub skip_step {
-    my ( $self, $context, $type, $reason ) = @_;
+    my ( $self, $context, $type, $reason, $redispatch ) = @_;
+
+    my $step_name = $redispatch ? 'sub_step' : 'step';
+    my $step_done_name = $step_name . '_done';
 
     # Pretend to start step execution
-    $context->harness->step($context);
+    $context->harness->$step_name($context);
 
     # Create a result object
     my $result = Test::BDD::Cucumber::Model::Result->new(
@@ -559,8 +654,8 @@ sub skip_step {
     );
 
     # Pretend we executed it
-    $context->harness->add_result($result);
-    $context->harness->step_done( $context, $result );
+    $context->harness->add_result($result) unless $redispatch;
+    $context->harness->$step_done_name( $context, $result );
     return $result;
 }
 
