@@ -1,5 +1,5 @@
 package Test::BDD::Cucumber::StepContext;
-$Test::BDD::Cucumber::StepContext::VERSION = '0.31';
+$Test::BDD::Cucumber::StepContext::VERSION = '0.45';
 use Moose;
 use List::Util qw( first );
 
@@ -9,7 +9,7 @@ Test::BDD::Cucumber::StepContext - Data made available to step definitions
 
 =head1 VERSION
 
-version 0.31
+version 0.45
 
 =head1 DESCRIPTION
 
@@ -78,10 +78,16 @@ objects respectively.
 
 =cut
 
-has 'feature' =>
-  ( is => 'ro', required => 1, isa => 'Test::BDD::Cucumber::Model::Feature' );
-has 'scenario' =>
-  ( is => 'ro', required => 1, isa => 'Test::BDD::Cucumber::Model::Scenario' );
+has 'feature' => (
+    is       => 'ro',
+    required => 1,
+    isa      => 'Test::BDD::Cucumber::Model::Feature'
+);
+has 'scenario' => (
+    is       => 'ro',
+    required => 1,
+    isa      => 'Test::BDD::Cucumber::Model::Scenario'
+);
 has 'step' =>
   ( is => 'ro', required => 0, isa => 'Test::BDD::Cucumber::Model::Step' );
 
@@ -111,6 +117,20 @@ The L<Test::BDD::Cucumber::Harness> harness being used by the executor.
 has 'harness' =>
   ( is => 'ro', required => 1, isa => 'Test::BDD::Cucumber::Harness' );
 
+=head2 executor
+
+Weak reference to the L<Test::BDD::Cucumber::Executor> being used - this allows
+for step redispatch.
+
+=cut
+
+has 'executor' => (
+    is       => 'ro',
+    required => 1,
+    isa      => 'Test::BDD::Cucumber::Executor',
+    weak_ref => 1
+);
+
 =head2 matches
 
 Any matches caught by the Step Definition's regex. These are also available as
@@ -154,6 +174,15 @@ this step which is actually an internal hook, i.e. a Before or After step
 has 'is_hook' =>
   ( is => 'ro', isa => 'Bool', lazy => 1, builder => '_build_is_hook' );
 
+=head2 parent
+
+If a step redispatches to another step, the child step will have a link back to
+its parent step here; otherwise undef. See L</Redispatching>.
+
+=cut
+
+has 'parent' => ( is => 'ro', isa => 'Test::BDD::Cucumber::StepContext' );
+
 =head1 METHODS
 
 =head2 background
@@ -249,6 +278,101 @@ sub transform {
     return $value;
 }
 
+=head1 Redispatching
+
+Sometimes you want to call one step from another step. You can do this via the
+I<StepContext>, using the C<dispatch()> method. For example:
+
+  Given qr/I have entered (\d+)/, sub {
+        C->dispatch( 'Given', "I have pressed $1");
+        C->dispatch( 'Given', "I have pressed enter", { some => 'data' } );
+  };
+
+You redispatch step will have its own, new step context with almost everything
+copied from the parent step context. However, specifically not copied are:
+C<columns>, C<data>, the C<step> object, and of course the C<verb> and the
+C<text>.
+
+If you want to pass data to your child step, you should IDEALLY do it via the
+text of the step itself, or failing that, through the scenario-level stash.
+Otherwise it'd make more sense just to be calling some subroutine... But you
+B<can> pass in a third argument - a hashref which will be used as C<data>.
+
+If the step you dispatch to doesn't pass for any reason (can't be found, dies,
+fails, whatever), it'll throw an exception. This will get caught by the parent
+step, which will then fail, and show debugging output.
+
+B<You must use the English names for the step verb, because we have no access to
+the parser. Also, remember to quote them as if you're in a step file, there may
+be a subroutine defined with the same name.>
+
+=head2 dispatch
+
+    C->dispatch( 'Then', "the page has loaded successfully");
+
+See the paragraphs immediately above this
+
+=cut
+
+sub dispatch {
+    my ( $self, $verb, $text, $data ) = @_;
+
+    my $step = Test::BDD::Cucumber::Model::Step->new(
+        {
+            text => $text,
+            verb => $verb,
+            line => Test::BDD::Cucumber::Model::Line->new(
+                {
+                    number      => $self->step->line->number,
+                    raw_content => "[Redispatched step: $verb $text]",
+                    document    => $self->step->line->document,
+                }
+            ),
+        }
+    );
+
+    my $columns;
+    if ($data) {
+        if ( ref $data eq 'HASH' ) {
+            $columns = [ sort keys %$data ];
+        }
+    }
+
+    my $new_context = $self->new(
+        {
+            executor => $self->executor,
+            ( $data    ? ( _data   => $data )    : () ),
+            ( $columns ? ( columns => $columns ) : () ),
+            stash => {
+                feature  => $self->stash->{'feature'},
+                scenario => $self->stash->{'scenario'},
+                step     => {},
+            },
+            feature      => $self->feature,
+            scenario     => $self->scenario,
+            harness      => $self->harness,
+            transformers => $self->transformers,
+
+            step => $step,
+            verb => lc($verb),
+            text => $text,
+        }
+    );
+
+    my $result = $self->executor->find_and_dispatch( $new_context, 0, 1 );
+
+    # If it didn't pass, short-circuit the rest
+    unless ( $result->result eq 'passing' ) {
+        my $error = "Redispatched step didn't pass:\n";
+        $error .= "\tStatus: " . $result->result . "\n";
+        $error .= "\tOutput: " . $result->output . "\n";
+        $error .= "Failure to redispatch a step causes the parent to fail\n";
+        die $error;
+    }
+
+    return $result;
+}
+
 # the builder for the is_hook attribute
 sub _build_is_hook {
     my $self = shift;
